                         {% if is_resource(pores) %}
                         <span about="{{ pores }}" typeof="{{ g.value(pores, RDF.type) }}">
                             {% if (pores, RDF.type, SDO.Organization) in g %}
                                <i class="fa fa-users"></i>&nbsp;
                             {% else %}
                                <i class="fa fa-user"></i>&nbsp;
                             {% endif %}
                             {% if g.value(pores, SDO.url) %}
                                <a rel="schema:url" href="{{ g.value(pores, SDO.url) }}">
                             {% elif g.value(pores, SDO.email) %}
                                <a rel="schema:email" href="mailto:{{ g.value(pores, SDO.email) }}">
                             {% endif %}
                             {% if g.value(pores, SDO.givenName) %}
                                <span property="schema:givenName">{{ g.value(pores, SDO.givenName) }}</span>
                             {% endif %}
                             {% if g.value(pores, SDO.familyName)  %}
                                 <span property="schema:familyName">{{ g.value(pores, SDO.familyName) }}</span>
                             {% elif g.value(pores, SDO.name)  %}
                                 <span property="schema:name">{{ g.value(pores, SDO.name) }}</span>
                             {% endif %}
                             {% if g.value(pores, SDO.url) and g.value(pores, SDO.email) %}
                                 </a> &lt;<a rel="schema:email" href="mailto:{{ g.value(pores, SDO.email) }}" class="email">{{ g.value(pores, SDO.email) }}</a>&gt;
                             {% elif g.value(pores, SDO.url) or g.value(pores, SDO.email) %}
                                 </a>
                             {% endif %}

                             {% if g.value(pores, SDO.affiliation) %}
                                 {% for affiliation_label, res3 in get_triples(g, pores, SDO.affiliation) %}
                                 <span property="schema:affiliation" class="affiliation">
                                     {% if res3 and g.value(res3, SDO.url) %}
                                     (<a href="{{ g.value(res3, SDO.url) }}">{{ affiliation_label }}</a>)
                                     {% elif affiliation_label %}
                                     <em>({{ affiliation_label }})</em>
                                     {% endif %}
                                 </span>
                                 {% endfor %}
                             {% endif %}

                             {% if g.value(pores, SDO.parentOrganization) %}
                                 {% for partof_label, res3 in get_triples(g, pores, SDO.parentOrganization) %}
                                 <span property="schema:affiliation">
                                     {% if res3 and g.value(res3, SDO.url) %}
                                     , <a href="{{ g.value(res3, SDO.url) }}">{{ partof_label }}</a>
                                         {% if g.value(res3, SDO.parentOrganization) %}
                                             {% for partof_label2, res4 in get_triples(g, res3, SDO.parentOrganization) %}
                                                 {% if res4 and g.value(res4, SDO.url) %}
                                                     , <a href="{{ g.value(res4, SDO.url) }}">{{ partof_label2 }}</a>
                                                 {% else %}
                                                     , <em>({{ partof_label2 }})</em>
                                                 {% endif %}
                                             {% endfor %}
                                         {% endif %}
                                     {% else %}
                                     , <em>({{ partof_label }})</em>
                                     {% endif %}
                                 </span>
                                 {% endfor %}
                             {% endif %}

                             {% if g.value(pores, SDO.partOf) %}
                                 {% for partof_label, res3 in get_triples(g, pores, SDO.partOf) %}
                                 <span property="schema:affiliation">
                                     {% if res3 and g.value(res3, SDO.url) %}
                                     , <a href="{{ g.value(res3, SDO.url) }}">{{ partof_label }}</a>
                                     {% else %}
                                     , <em>({{ partof_label }})</em>
                                     {% endif %}
                                 </span>
                                 {% endfor %}
                             {% endif %}
                         </span>
                         {% elif pores %}
                            {{ pores }}
                         {% elif label %}
                            {{ label }}
                         {% endif %}
