<div class="reference" about="{{ ref }}" typeof="{{ g.value(ref, RDF.type ) }}">

<ul>
     {% for label, aures in get_triples(g, ref, SDO.author) %}
     <li property="schema:author">
         {% include 'reference_author.html' %}
     </li>
     {% endfor %}
</ul>

{% if g.value(ref, SDO.datePublished) %}
(<span property="schema:datePublished">{{ g.value( ref , SDO.datePublished) }}</span>)
{% elif g.value(ref, SDO.isPartOf) and g.value( g.value(ref, SDO.isPartOf) , SDO.datePublished) %}
(<span property="schema:datePublished">{{ g.value( g.value(ref, SDO.isPartOf) , SDO.datePublished) }}</span>)
{% endif %}


{% if g.value(ref, SDO.url) %}
<a href="{{ g.value(ref, SDO.url) }}" rel="schema:url">
{% endif %}
<span property="schema:name"><strong>{{ g.value(ref, SDO.name) }}</strong></span>
{% if g.value(ref, SDO.url) %}
</a>
{% endif %}

{% if g.value(ref, SDO.isPartOf) and g.value( g.value(ref, SDO.isPartOf) , SDO.datePublished) %}
    {% set parentref = g.value(ref, SDO.isPartOf) %}
    <span about="{{ parentref }}" typeof="{{ g.value(parentref, RDF.type) }}">
    . In:
    {% if g.value(parentref, SDO.name) %}
    <span property="schema:name">{{ g.value(parentref, SDO.name) }}</span>
    {% endif %}
    {% if g.value(parentref, SDO.issue) %}
    , <span property="schema:issue" title="The issue (e.g. journal/proceedings) in which this publication has been published">{{ g.value(parentref, SDO.issue) }}</span>
    {% endif %}
    </span>
{% endif %}

{% if g.value(ref, SDO.pageStart)  and g.value(ref, SDO.pageEnd) %}
, <span property="schema:pageStart" title="page number where this publication can be found">{{ g.value(ref, SDO.pageStart) }}</span>-<span property="schema:pageEnd">{{ g.value(ref, SDO.pageEnd) }}</span>
{% elif g.value(ref, SDO.pagination) %}
, <span property="schema:pagination" title="page number where this publication ends">{{ g.value(ref, SDO.pagination) }}</span>
{% endif %}


{% set doi = get_doi(g,ref) %}
{% if doi %}
DOI: <a href="https://doi.org/{{ doi }}" property="schema:identifier">{{ doi }}</a>
{% endif %}

</div>
