<tr about="{{ targetres }}" typeof="{{ g.value(targetres, RDF.type) }}" class="targetProduct {{ get_last_component(g.value(targetres, RDF.type)) }}">
    <td class="name" property="schema:name">
        {% if (targetres, SDO.name, None) in g %}
        {% if res != targetres %}
        <a href="{{ res }}">{{ g.value(targetres, SDO.name) }}</a>
        {% else %}
        {{ g.value(targetres, SDO.name) }}
        {% endif %}
        {% endif %}
    </td>
    <td class="version" property="schema:version">
        {% if  (targetres, SDO.version, None) in g %}
        {{ g.value(targetres, SDO.version) }}
        {% endif %}
    </td>
    <td class="interface">
        <ul>
        {% for label, comment in get_interface_types(g, targetres, contextgraph,fallback=True) %}
        <li{% if comment %} title="{{ comment }}"{% endif %}{% if label == "Unknown" %} class="negative"{% endif %}>{{ label }}</li>
        {% endfor %}
        </ul>
    </td>
    <td class="description" property="schema:description">
        {% if  (targetres, SDO.description, None) in g %}
        {% if g.value(targetres, SDO.description) != g.value(res, SDO.description) %}
        <em>{{ g.value(targetres, SDO.description) }}</em>
        {% endif %}
        {% endif %}
    </td>
    <td class="links">
        <ul>
        {% for _,_, value in g.triples((targetres, SDO.url, None)) %}
        {% if (targetres, RDF.type, SDO.WebApplication) in g or (targetres, RDF.type, SDO.WebSite) in g or (targetres, RDF.type, SDO.WebPage) in g or (targetres, RDF.type, STYPE.NotebookApplication) in g %}
        <li><i class="fa fa-globe"></i>&nbsp;<a rel="schema:url" class="mainlink" href="{{ value }}" title="For webapplications and webservices, this is where you can access the software as a service on the web">{{ value }}</a></li>
        {% elif (targetres, RDF.type, SDO.WebAPI) in g %}
        <li><i class="fa fa-cog"></i>&nbsp;<a rel="schema:url" href="{{ value }}" title="For webservices, this is where you can access the software as a service on the web">{{ value }}</a></li>
        {% endif %}
        {% endfor %}
        </ul>
    </td>
    <td class="developmentStatus">
        {% if (targetres, CODEMETA.developmentStatus, None) in g %}
            <ul>
                {% for _,_, devstatres in g.triples((targetres, CODEMETA.developmentStatus, None)) %}
                {% include 'developmentstatus.html' %}
                {% endfor %}
            </ul>
        {% endif %}
    </td>
    <td class="maintainer">
        <ul>
         {% for label, pores in get_triples(g, targetres, SDO.maintainer) %}
         <li>{% include 'person_or_org_minimal.html' %}</li>
         {% endfor %}
        </ul>
    </td>
    <td class="author">
        <ul>
         {% for label, pores in get_triples(g, targetres, SDO.author) %}
         <li>{% include 'person_or_org_minimal.html' %}</li>
         {% endfor %}
        </ul>
    </td>
    <td class="producer">
        <ul>
         {% for label, pores in get_triples(g, targetres, SDO.producer) %}
         <li>{% include 'person_or_org.html' %}</li>
         {% endfor %}
        </ul>
    </td>
</tr>
