body {
    font-family: sans-serif;
    font-size: 0.8em;
    margin: 0px;
    padding: 0px;
}
div#header {
    margin: 0px;
    padding: 1px;
    background: #000c48;
    color: white;
    text-align: center;
}
div#header h1 {
    margin: 0px;
    padding: 0px;
    margin-top: 5px;
    margin-bottom: 5px;
    color: white;
}
div#navbar {
    margin: 0px;
    padding: 3px;
    color: #000c48;
    background: white;
    margin-bottom: 3px;
    vertical-align: middle;
}
nav {
    float: right;
    text-align: right;
}
nav a, nav a:visited {
    color: #000c48;
}
div#navbar div#search button.search {
    display: inline-block;
    width: 24px;
    height: 24px;
    background: transparent;
}
div#navbar div#search input {
    margin: 0px;
}

div.leftcolumn {
    flex: 60%;
    padding: 10px;
}

div.rightcolumn {
    flex: 40%;
}

@media screen and (max-width: 600px) {
    div.leftcolumn div.rightcolumn {
        flex: 100%;
    }
}


body.page div.softwaresourcecode {
    display: flex;
}

body.page div.softwaresourcecode div.properties {
    background: #e4e4e4;
    padding: 10px;
    border-radius: 25px;
}

dt {
    cursor: help;
    font-weight: bold;
}

dd ul, table ul {
    list-style: none;
    margin: 0px;
    padding: 0px;
}

.keywords ul, .softwareRequirements ul, .category ul li {
    list-style: none;
}

.keywords ul li, .softwareRequirements ul li, .category ul li, .targetplatforms ul li {
    display: inline-block;
    padding: 2px;
    margin: 2px;
    border: #aac 1px solid;
    background: white;
    color: black;
    font-size: 0.9em;
}


.interface ul, table ul {
    list-style: none;
}

.interface ul li {
    display: inline-block;
    padding: 2px;
    margin: 2px;
    border: #aac 1px solid;
    background: white;
    color: black;
    font-size: 0.9em;
    cursor: help;
}
div.properties {
    margin-top: 10px;
}


div.go {
    margin-top: 10px;
    padding: 10px;
    background: #e1275e;
    text-align: center;
    border-radius: 25px;
    color: white;
    font-weight: bold;
    clear: both;
}
div.go a {
    color: white;
    font-weight: bold;
}
div.go i {
    font-size: 200%;
}
div.go strong {
    font-size: 125%;
}
div.go tt {
    display: block;
    font-size: 90%;
    font-weight: normal;
}
div.go:hover {
    background: #bb0138;
}

div.targetProduct {
    width: 80%;
    margin-left: auto;
    margin-right: auto;
    border-radius: 25px;
    padding-left: 10px;
    padding-right: 10px;
}


div.targetProduct, tr.targetProduct td {
    background: rgb(211, 220, 255);
}
body div.CommandLineApplication, table tr.CommandLineApplication td {
    background: #dedede;
}
body div.WebAPI, table tr.WebAPI td {
    background: #e2f8f8
}

tr.targetProduct td.name {
    text-align: right;
}

div.targetProduct div.properties {
    background: transparent;
    margin-top: 0px;
    text-align: left;
}
tr.targetProduct td.links a.mainlink {
    font-weight: bold;
    border-bottom: #2628c9 2px solid;
    text-decoration: none;
}
tr.targetProduct td.developmentStatus, tr.targetProduct td.author {
    /* these fields are typically empty */
    background: white;
}
div.targetProduct h2 {
    margin-left: 20px;
    color: #bb0138;
    padding-top: 10px;
    text-align: center;
}
img.thumbnail {
    float: left;
    max-width: 120px;
}
.leftcolumn img.thumbnail {
    padding: 10px;
}
.positive {
    color: green;
}
.negative, .negative a {
    color: #da4800 !important;
}
div.warning {
    color: white;
    padding: 2px;
    border: 1px solid red;
    background: #672525;
    text-align: center;
}
div#footer {
    padding: 10px;
    text-align: center;
    font-family: monospace;
    color: #333;
    font-size: 80%;
}

table {
    vertical-align: top;
}
table td.name {
    font-weight: bold;
}
th {
    border-bottom: 1px solid black;
    text-align: left;
    vertical-align: top;
}
tr.softwaresourcecode td {
    border-top: 1px solid black;
    text-align: left;
    vertical-align: top;
}
td {
    border-right: 1px solid #eee;
}

th.name, td.name {
    width: 10%;
}
th.description, td.description {
    width: 33%;
}
th.maintainer, th.author, th.producer {
    width: 8%;
}
div.keywords, div.category, div.targetplatforms {
    margin-top: 5px;
    font-size: 90%;
}
.interface {
    font-size: 90%;
}
div.keywords ul, div.category ul, div.targetplatforms ul {
    display: inline;
}
div.keywords ul li, .interface ul li, dd.softwareRequirements ul li, div.category ul li, div.targetplatforms ul li {
    background: #ddd;
    font-size: 90%;
}
div.properties .email, div.properties .affiliation, td .email, td .affiliation {
    font-size: 80%;
}
tr.softwaresourcecode span.dateModified {
    display: block;
    font-family: monospace;
    font-size: 75%;
    cursor: help;
}
dd.executableName {
    font-family: monospace;
}
div.reference {
    display: inline;
}
div.reference ul {
    list-style: none;
    margin: 0px;
    padding: 0px;
}
div.reference ul, div.reference ul li {
    display: inline;
    font-style: italic;
}
div.reference ul li:after { content: "& "; }
div.reference ul li:last-child:after { content: ""; }

body.cards {
    background: #d1dbe1 url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAgAAAAICAYAAADED76LAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAD1JREFUeNpiYmBgMAZihv///2PFjAxQAOKgA0ZGRmMGZAXIGGYyVmPhknh1YlOALgkWwycJV4BLEoQBAgwA2xBhP01jk8EAAAAASUVORK5CYII=) 0 0 repeat fixed;
}

a {
    text-decoration: none;
}
a, a:visited {
    color: #2350ff;
}
a:hover {
    text-decoration: underline;
    color: #bb0138;
}

div.card {
    min-width: 320px;
    max-width: 640px;
    border: 1px solid #cbdfe5;
    border-radius: 20px;
    background: #fff;
    padding: 10px;
    text-align: center;
    margin-bottom: 15px;
    margin-left: auto;
    margin-right: auto;
}
div.card h2 {
    color: #000c48;
    margin-bottom: 2px;
    margin-top: 0px;
}
div.card h2 span.version {
    font-size: 60%;
    font-weight: normal;
}
div.card div.topbar {
    display: flex;
    margin: 0px;
}
div.card div.interface {
    flex: 50%;
    display: inline-block;
    text-align: left;
    font-size: 80%;
    opacity: 0.75;
}
div.card div.developmentStatus {
    flex: 50%;
    display: inline-block;
    text-align: right;
    font-size: 80%;
    opacity: 0.75;
}
div.card div.keywords, div.card div.category, div.card div.targetplatforms {
    text-align: left;
}
div.card div.keywords li, div.card div.category li, div.card div.targetplatforms li {
    color: #666;
    border: none;
}
div.card ul {
    margin: 0px;
    padding: 0px;
}
div.card div.interface li {
    border-radius: 25px;
}
div.card div.description {
    margin-top: 5px;
    margin-bottom: 5px;
}
div.card div.minilinks {
    float: right;
}
div.card div.links {
    margin-top: 2px;
    clear: both;
}
div.card div.links li {
    list-style: none;
}
div.card div.links li {
    display: block;
    border-radius: 5px;
    margin-bottom: 2px;
    background: #cbdfe5;
    padding: 3px;
}
div.card div.links a {
    font-weight: bold;
}
div.card div.links li {
    border: 1px solid #cbdfe7;
}
div.card div.links li:hover {
    border: 1px #bb0138 solid;
}
div.card div.links li.go {
    background: #2350ff;
}
div.card div.links li:hover {
    background: #bb0138;
}
div.card div.links li.go span.description {
    color: #ccc;
}
div.card div.links li.go a, div.card div.links li.go a:visited, div.card div.links li:hover, div.card div.links li:hover a {
    color: white;
}
div.card div.author ul, div.card div.producer ul, div.provider ul {
    list-style: none;
}
div.card div.author li, div.card div.producer li, div.provider li {
    display: inline-block;
    font-size: 90%;
}
div.card div.author li, div.card div.producer li, div.card div.author li a, div.card div.producer li a, div.card div.provider li a {
    color: #0092bc;
}

div.card div.dateCreated {
    display: inline;
    font-size: 80%;
    text-align: center;
    color: #666;
}
div.card div.dateModified {
    display: inline;
    font-size: 80%;
    text-align: center;
    color: #666;
}
div card div.links li>a {
    font-size: 110%;
}
div.card div.links li tt {
    display: block;
    font-size: 80%;
}
div.card div.links li span.description {
    display: block;
    font-style: italic;
    font-size: 80%;
}
div.log {
    margin-top: 10px;
}
#harvestlog {
    display: none;
}
pre#harvestlog {
    word-wrap: break-word;
    white-space: pre-wrap;
}
.rating0, .rating1, .rating0 a, .rating1 a {
    color: red;
}
.rating2, .rating2 a {
    color: #ff3400;
}
.rating3, .rating3 a {
    color: #ffad00;
}
.rating4, .rating4 a {
    color: #5a7410;
}
.rating5, .rating5 a {
    color: green;
}

dd.iodata div {
    border-bottom: 1px solid #999;
    border-left: 1px solid #999;
    padding: 5px;
}
.iodata span.format {
    display: inline-block;
    font-family: monospace;
    margin-right: 10px;
}
.iodata span.type {
    display: inline-block;
    font-family: monospace;
    margin-right: 10px;
}

div.card img.thumbnail {
    display: block;
    float: none;
    margin-left: auto;
    margin-right: auto;
    text-align: center;
}
li.maintainer {
    font-weight: bold;
}
div.card div.topbar div.warning {
    display: none;
}
div.card div.topbar ul {
    list-style: none;
}
div.group>h2 {
    display: block;
    min-width: 320px;
    max-width: 640px;
    text-align: left;
    color: #000c48;
    margin-left: auto;
    margin-right: auto;
}
div.group {
    border-top: 1px #000c48 solid;
    border-bottom: 1px #000c48 solid;
    border-left: 10px #000c48 solid;
    background: #00000020;
    margin-bottom: 5px;
}
table td.group {
    text-align: left;
    margin-left: 20px;
    font-weight: bold;
    background: #000c48;
    color: white;
    height: 5px;
}
body.page div.cards div.card {
    display: block;
    border: 1px solid black;
}
div.screenshots img {
    max-width: 800px;
}
