        <div id="footer">
            This page was automatically generated by <a href="https://github.com/proycon/codemetapy">codemetapy {{ get_version("codemetapy") }}</a>{% if toolstore %} and <a href="https://github.com/proycon/codemeta-server">codemeta-server {{ get_version("codemeta_server") }}</a> {% else %} at {{ now }}{% endif %}
            {% if baseurl and toolstore %}
            on the basis of <a href="{{ baseurl }}data.json" title="This file is the knowledge graph containing the complete metadata of all tools, in JSON-LD format">this metadata (JSON-LD)</a> <a href="{{ baseurl }}data.ttl" title="This file is the knowledge graph containing the complete metadata of all tools, in Turtle format">(turtle)</a>.<br />A <a href="{{ baseurl }}api/sparql/">SPARQL endpoint</a> <a href="{{ baseurl }}api/">(+ web interface)</a> is also available.
            {% endif %}.
        </div>
