      {% for _,_,review in g.triples((sscres, SDO.review,None)) %}
        {% if str(g.value(review, SDO.name)).startswith("Automatic software metadata validation report") %}
            {% if g.value(review, SDO.reviewRating) %}
            {% set rating = int(g.value(review, SDO.reviewRating)) %}
            <span class="rating{{ rating }}" title="Automatic software metadata rating, click to inspect validation report">
            <a href="{{ review }}">
            {% for _ in range(0, rating) %}
            &starf;
            {% endfor %}
            {% for _ in range(rating, 5) %}
            &star;
            {% endfor %}
            </a>
            </span>
            {% else %}
            <span class="rating0" title="Automatic software metadata rating, click to inspect validation report">
            <a href="{{ review }}">
            &star;
            &star;
            &star;
            &star;
            &star;
            </a>
            </span>
            {% endif %}
        {% endif %}
      {% endfor %}
