                         {% if aures %}
                         <span about="{{ aures }}" typeof="{{ g.value(aures, RDF.type) }}">
                             {% if g.value(aures, SDO.url) %}
                                <a rel="schema:url" href="{{ g.value(aures, SDO.url) }}">
                             {% elif g.value(aures, SDO.email) %}
                                <a rel="schema:email" href="mailto:{{ g.value(aures, SDO.email) }}">
                             {% endif %}
                             {% if g.value(aures, SDO.familyName)  %}
                                 <span property="schema:familyName">{{ g.value(aures, SDO.familyName) }}</span>,
                                 <span property="schema:givenName">{{ g.value(aures, SDO.givenName) }}</span>
                             {% elif g.value(aures, SDO.name)  %}
                                 <span property="schema:name">{{ g.value(aures, SDO.name) }}</span>
                             {% else %}
                                {{ aures }}
                             {% endif %}
                             {% if g.value(aures, SDO.url) and g.value(aures, SDO.email) %}
                                 </a> &lt;<a rel="schema:email" href="mailto:{{ g.value(aures, SDO.email) }}">{{ g.value(aures, SDO.email) }}</a>&gt;
                             {% elif g.value(aures, SDO.url) or g.value(aures, SDO.email) %}
                                 </a>
                             {% endif %}
                         </span>
                         {% elif label %}
                            {{ label }}
                         {% endif %}
