        <div id="footer">
            This page was automatically generated by <a href="https://github.com/proycon/codemetapy">codemetapy {{ get_version("codemetapy") }}</a>{% if toolstore %} and <a href="https://github.com/proycon/codemeta-server">codemeta-server {{ get_version("codemeta_server") }}</a> {% else %} at {{ now }}{% endif %}
            {% if baseurl and toolstore %}
                on the basis of <a href="{{ res }}.json" title="This is a JSON-LD file following using the codemeta/schema.org vocabulary, it should also be retrievable through content negotiation">this metadata (JSON-LD)</a>,
                <a href="{{ res }}.ttl" title="This is a Turtle file using the codemeta/schema.org vocabulary, it should also be retrievable through content negotiation">(turtle)</a>.<br/>
                A <a href="{{ baseurl }}api/sparql/">SPARQL endpoint</a>
                <a href="{{ baseurl }}api/">(+ web interface)</a> is also available.
            {% endif %}
        </div>
