<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:schema="http://schema.org/" xmlns:codemeta="https://codemeta.github.io/terms/" xmlns:stype="https://w3id.org/software-types/" prefix="og: https://ogp.me/ns# schema: http:/schema.org codemeta: https://codemeta.github.io/terms/">
    <head>
        {% include "head.html" %}
    </head>
    <body>
        {% include "header.html" %}
        {% include "intro.html" %}
        <table>
            <thead>
                <th class="name">Name</th>
                <th class="version">Version</th>
                <th class="interfaces" title="Type of interface that this software offers">Interface type</th>
                <th class="description">Description</th>
                <th class="links" title="Source code repository, project website or service endpoint">Links</th>
                <th class="developmentStatus" title="Development status">Status</th>
                <th class="maintainer" title="The maintainer is the primary contact person">Maintainer</th>
                <th class="author" title="The authors/developers of the software">Authors</th>
                <th class="producer" title="The institute or person who produces the software; or for services the institute or person who provides the service">Producer/Provider</th>
            </thead>
            <tbody>
        {% for group, explicit, resources in index %}
        {% if explicit and resources %}
        <tr>
            <td class="group" colspan="9">{{ group }}</td>
        </tr>
        {% endif %}
        {% for res, label in resources %}
        {% include 'item_softwaresourcecode.html' %}
        {% endfor %}
        {% if explicit and resources %}
        <tr>
            <td class="group" colspan="9"></td>
        </tr>
        {% endif %}
        {% endfor %}
            </tbody>
        </table>
        {% include 'indexfooter.html' %}
    </body>
</html>
