<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:schema="https://schema.org/" xmlns:codemeta="https://doi.org/10.5063/schema/codemeta-2.0/" xmlns:stype="https://w3id.org/software-types/">
    <head>
        {% include "head.html" %}
    </head>
    <body>
        {% set targetres = res %}
        {% include "targetproduct.html" %}
        {% include "footer.html" %}
    </body>
</html>
