<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:schema="http://schema.org/" xmlns:codemeta="https://codemeta.github.io/terms/" xmlns:stype="https://w3id.org/software-types/" prefix="og: https://ogp.me/ns# schema: http:/schema.org codemeta: https://codemeta.github.io/terms/">
    <head>
        {% include "head.html" %}
    </head>
    <body class="cards">
        {% include "header.html" %}
        {% include "intro.html" %}
        {% for group, explicit, resources in index %}
        {% for res, label in resources %}
         {% for _, targetres in get_triples(g, res, SDO.targetProduct, abcsort=True) %}
           {% if ((targetres, RDF.type, SDO.WebApplication) in g or
             (targetres, RDF.type, SDO.WebSite) in g or
             (targetres, RDF.type, SDO.NotebookApplication) in g or
             (targetres, RDF.type, SDO.WebPage) in g) and (targetres, SDO.url,None) in g %}
             {% include 'card_service.html' %}
           {% endif %}
         {% endfor %}
        {% endfor %}
        {% endfor %}
        {% include 'indexfooter.html' %}
    </body>
</html>
