<!DOCTYPE html>
<html xmlns="http://www.w3.org/1999/xhtml" xmlns:schema="http://schema.org/" xmlns:codemeta="https://codemeta.github.io/terms/" xmlns:stype="https://w3id.org/software-types/" prefix="og: https://ogp.me/ns# schema: http:/schema.org codemeta: https://codemeta.github.io/terms/">
    <head>
        {% include "head.html" %}
    </head>
    <body class="cards">
        {% include "header.html" %}
        {% include "intro.html" %}
        {% for group, explicit, resources in index %}
        {% if explicit %}
        <div class="group">
            <h2>{{ group }}</h2>
        {% endif %}
        {% for res, label in resources %}
        {% include 'card_softwaresourcecode.html' %}
        {% endfor %}
        {% if explicit %}
        </div>
        {% endif %}
        {% endfor %}
        {% include 'indexfooter.html' %}
    </body>
</html>
