                {% if url %}
                <a href="{{ url }}"><img src="{{ img }}" title="{{ label }} " alt=" {{ label }}" class="badge" /></a>
                {% else %}
                <img src="{{ img }}" title="{{ label }} " alt=" {{ label }}" class="badge" />
                {% endif %}
